/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file class.cpp
 * @brief Class implementation file
 */

#include "bzrh.h"
#include "../botkernel.h"

/**
 * Constructor
 */ 
BZRH::BZRH(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugin that allows queries to bugzilla.redhat.com";
	this->version = "0.1.1";
	this->name = "bzrh";
   this->bindFunction("bzsearch",IN_COMMAND_HANDLER,"bzsearch",0,120);
   this->bindFunction("bug",IN_COMMAND_HANDLER,"bug",0,60);
   this->bindFunction("",IN_ALL_MSGS,"checkBug",0,60);
}

/**
 * Search for bugs on bugzilla
 * @param pattern Search pattern
 * @param max max results
 * @return vector containing results
 */
vector<string> BZRH::searchBugs(string pattern,string max) 
{
   string bug_search = Tools::urlencode(pattern);
   const string URL_BZRH = "https://bugzilla.redhat.com/";
   const string BEGIN_BUGS = "</thead>";
   const string END_BUGS = "</table>";
   const string BEGIN_DETAIL = "<tr class=\"";
   const string BEGIN_URL = "<a href=\"";
   const string BEGIN_DESC = "<span title=\"";
   vector<string> results ;
   string::size_type pos;
   string tmp;
   string URL = URL_BZRH+"buglist.cgi?bug_status=NEW&bug_status=VERIFIED&bug_status=ASSIGNED&bug_status=MODIFIED&bug_status=NEEDINFO&bug_status=REOPENED&bug_status=ON_DEV&bug_status=ON_QA&bug_status=FAILS_QA&bug_status=POST&bug_status=RELEASE_PENDING&field0-0-0=product&type0-0-0=substring&value0-0-0="+bug_search+"&field0-0-1=component&type0-0-1=substring&value0-0-1="+bug_search+"&field0-0-2=short_desc&type0-0-2=substring&value0-0-2="+bug_search+"&field0-0-3=status_whiteboard&type0-0-3=substring&value0-0-3="+bug_search;
   CURL *curl;  
   CURLcode result;  
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init();  
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, BZRH::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);  
   
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
 
      if (result == CURLE_OK) {  
         pos = buffer.find(BEGIN_BUGS);
         if ( pos == string::npos ) {
            results.clear();
            results.push_back("Zarro Boogs found."); 
            return results;
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(END_BUGS);
         if ( pos == string::npos ) {
            results.clear();
            results.push_back("* Parse error *"); 
            return results;
         }
         buffer = buffer.substr(0,pos);
         pos= buffer.find(BEGIN_DETAIL);
         while(pos != string::npos ) {
            pos = buffer.find(BEGIN_URL);
            if ( pos == string::npos ) {
               results.clear();
               results.push_back("* Parse error *"); 
               return results;
            }
            buffer = buffer.substr(pos+BEGIN_URL.length());
            tmp = URL_BZRH+buffer.substr(0,buffer.find("\">"));
            for(unsigned int i = 0 ; i < 8 ; i ++ ) {
               pos = buffer.find(BEGIN_DESC);
               if ( pos == string::npos ) {
                  results.clear();
                  results.push_back("* Parse error *"); 
                  return results;
               }
               buffer=buffer.substr(pos+BEGIN_DESC.length());
            }
            results.push_back(tmp+" : "+Tools::cleanHTML(buffer.substr(0,buffer.find("\">"))));
            pos= buffer.find(BEGIN_DETAIL);
         }
         if ( (max=="") || (max=="0") || (Tools::strToUnsignedInt(max)>=results.size()) ) {
            return results;
         }
         else {
            tmp = "Too many bugs ("+Tools::intToStr(results.size())+") ! Max is "+max+". Use bz's webiste : "+URL_BZRH ;
            results.clear();
            results.push_back(tmp); 
            return results;
         }
      }  
      else {   
         results.clear();
         results.push_back("* ERROR, try again later please *");
         return results;
      }  
   }
   results.clear();
   results.push_back("curl init error");
   return results;
}

/**
 * Retrieve informations about a bug
 * @param bug Bug number
 * @return Bug's infos
 */
string BZRH::getBugInfos(string bug) 
{
   string bug_number;
   if (bug[0] == '#' ) {
      bug_number = bug.substr(1);
   }
   else {
      bug_number =bug;
   }
   const string URL = "https://bugzilla.redhat.com/long_list.cgi?buglist="+Tools::urlencode(bug_number);
   const string SHOW_BUG_URL = "http://bugzilla.redhat.com/show_bug.cgi?id=";
   const string BEGIN_VALUE= "<td>";
   const string BEGIN_VALUE_SUMMARY = "<td colspan=\"7\">";
   const string END_VALUE = "&nbsp;";
   const string SEVERITY = "<b>Severity:</b>";
   const string PRIORITY = "<b>Priority:</b>";
   const string ASSIGNED = "<b>Assigned To:</b>";
   const string STATUS   = "<b>Status:</b>";
   const string SUMMARY  = "<b>Summary:</b>";
   string severity;
   string priority;
   string assigned;
   string status;
   string summary;
   string::size_type pos;
   CURL *curl;  
   CURLcode result;  
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init();  
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, BZRH::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);  
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
      if (result == CURLE_OK) {  
         pos = buffer.find(STATUS);
         if ( pos == string::npos ) {
            return "Bug not found";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_VALUE) +BEGIN_VALUE.length();
         status = buffer.substr(pos, buffer.find(END_VALUE) - pos );
         pos = buffer.find(SEVERITY);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_VALUE) +BEGIN_VALUE.length();
         severity = buffer.substr(pos, buffer.find(END_VALUE) - pos );
         pos = buffer.find(PRIORITY);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_VALUE) +BEGIN_VALUE.length();
         priority = buffer.substr(pos, buffer.find(END_VALUE) - pos );
         pos = buffer.find(ASSIGNED);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_VALUE) +BEGIN_VALUE.length();
         assigned = buffer.substr(pos, buffer.find(END_VALUE) - pos );
         pos = buffer.find(SUMMARY);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_VALUE_SUMMARY) +BEGIN_VALUE_SUMMARY.length();
         summary = buffer.substr(pos, buffer.find(END_VALUE) - pos );
         return ("bug "+SHOW_BUG_URL+bug_number+" : "+severity+", "+priority+", "+status+", "+Tools::cleanHTML(assigned)+", "+Tools::cleanHTML(summary));
      }  
      else {   
         return "* ERROR, try again later please *"; 
      }  
   }
   return "curl init error";   
}

int BZRH::writer(char *data, size_t size, size_t nmemb,string *buffer)  
{  
   int result = 0;  
   if (buffer != NULL) {  
      buffer->append(data, size * nmemb);  
      result = size * nmemb;  
   }   
   return result;  
}

extern "C"
{
	Plugin *contruct_bzrh(BotKernel*b)
	{
		return new BZRH(b);
	}
	void destroy_bzrh(Plugin*p)
	{
		delete p;
	}
	bool bzsearch (Message*m,Plugin*p,BotKernel*b)
	{
      BZRH*bz= (BZRH*)p;
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),bz->searchBugs(Tools::vectorToString(m->getSplit()," ",4), b->getCONFF()->getValue(p->getName()+".maxbugs"))));
      }
		return true;
	}
	bool bug (Message*m,Plugin*p,BotKernel*b)
	{
      BZRH*bz= (BZRH*)p;
      if ( m->isPublic() && (m->nbParts() == 5) && (m->getPart(4)!="") ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(4))));
      }
		return true;
	}
   bool checkBug(Message*m,Plugin*p,BotKernel*b)
   {
      const string BUG = "bug";
      const string URL1 = "https://bugzilla.redhat.com/show_bug.cgi?id=";
      const string URL2 = "http://bugzilla.redhat.com/show_bug.cgi?id=";
      BZRH*bz= (BZRH*)p;
      if (m->isPublic() && (b->getCONFF()->getValue(p->getName()+".autoreact")=="1")) {
         if ( m->getPart(3).find(":"+BUG) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(4))));
         }
         else if (m->getPart(3).find(":"+URL1) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(3).substr(URL1.length()+1))));
         }
         else if (m->getPart(3).find(":"+URL2) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(3).substr(URL2.length()+1))));
         }
         for (unsigned int i = 4 ; i < m->nbParts() ; i ++ ) {
            if ( m->getPart(i).find(BUG) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i+1))));
            }
            else if (m->getPart(i).find(URL1) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i).substr(URL1.length()))));
            }
            else if (m->getPart(i).find(URL2) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i).substr(URL2.length()))));
            }
         }  
      }
      return true;   
   }
}
