/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file fedorafr.cpp
 * @brief Fedorafr implementation file
 */

#include "fedorafr.h"
#include "../botkernel.h"

/**
 * Constructor
 */ 
Fedorafr::Fedorafr(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugins that allow people to search on Fedora-fr.org wiki and planet";
	this->version = "0.1.1";
	this->name = "fedorafr";
	this->bindFunction("wiki",IN_COMMAND_HANDLER,"wiki",0,10);
	this->bindFunction("planet",IN_COMMAND_HANDLER,"planet",0,10);
}



/**
 * Extract links in a wiki result page
 * @param datas HTML code from wiki page
 * @return Wiki's URLs 
 */
vector<string> Fedorafr::getWikiLinks(string datas) {
   vector<string> results;
   const string BASE_URL = "http://doc.fedora-fr.org" ;
   string::size_type pos;
   vector<string> split = Tools::stringToVector(datas,"\n",0) ;  
   for (unsigned int i = 0 ; i < split.size() ; i ++ ) {
      pos = split[i].find("href=\"");
      if ( pos != string::npos ) {
         results.push_back(BASE_URL+split[i].substr(pos+string("href=\"").size(),split[i].find(" title=\"")-(pos+string("href=\"").size()+1)));
      }
   }
   return results;
}

extern "C"
{
	Plugin *contruct_fedorafr(BotKernel*b)
	{
		return new Fedorafr(b);
	}
	void destroy_fedorafr(Plugin*p)
	{
		delete p;
	}
	bool wiki (Message*m,Plugin*p,BotKernel*b)
	{
      const string NO_RESULT_TITRE = "Aucun titre d’article ne correspond à la recherche.";
      const string NO_RESULT_TEXTE = "Aucun texte d’article ne correspond à la recherche.";
      const string RESULTS_TITRE = "Correspondances dans les titres d’articles";
      const string RESULTS_TEXTE = "Correspondances dans le texte d’articles";
      const string HEADLINE = "mw-headline";
      const string END_TITLE = "</span></h2>";
      string buffer = "";
      string titre = "";
      string::size_type pos=0;
      unsigned int startFrom=5;
      unsigned int nbResults ;
      vector<string> results;
      unsigned int maxResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".wiki_max_results"));
      Fedorafr*ffr = (Fedorafr*)p;
      Socket sock = Socket();
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         if (!sock.connectSock(80,"doc.fedora-fr.org","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to doc.fedora-fr.org *"));
            return true;
         }
         nbResults = Tools::strToInt(m->getPart(4));
         if (nbResults == 0 ) {
            nbResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".wiki_default_results"));  
            startFrom = 4;
         }
         else  if (nbResults > maxResults ) {
            nbResults = maxResults ;
         }
         sock.sendStr("GET /wiki/Special:Search?search="+Tools::urlencode(Tools::vectorToString(m->getSplit()," ",startFrom) )+"&fulltext=Rechercher HTTP/1.1\nHost: doc.fedora-fr.org\n\n");
         while (buffer.find("<form id=\"powersearch\"")==string::npos) {
            buffer += sock.receive() ;
         }
         pos = buffer.find(HEADLINE);
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result *"));
            return true;
         }
         buffer = buffer.substr(pos+string(HEADLINE).length()+2,buffer.length()-pos);
         titre = buffer.substr(0,buffer.find(END_TITLE) );
         buffer = buffer.substr(buffer.find(END_TITLE)+string(END_TITLE).length()+1); 
         pos = buffer.find(HEADLINE);
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result *"));
            return 0;
         }
         if ( titre == NO_RESULT_TITRE ) {
            // Nothing in titles, so we search in texts :
            buffer = buffer.substr(pos+string(HEADLINE).length()+2,buffer.length()-pos);
            titre = buffer.substr(0,buffer.find(END_TITLE) );
            buffer = buffer.substr(buffer.find(END_TITLE)+string(END_TITLE).length()+1); 
            pos = buffer.find("<p>");
            if ( pos == string::npos) {
	            b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result *"));
               return true;
            }
            if ( titre == NO_RESULT_TEXTE ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),"Aucun resultat."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"Rien dans les titres. Resultats dans les textes :"));
               results = ffr->getWikiLinks(buffer.substr(0,pos)) ;
               if (nbResults > results.size() ) {
                  nbResults = results.size();
               } 
               for ( unsigned int i = 0 ; i  < nbResults ; i ++ ) {
                  b->send(IRCProtocol::sendMsg(m->getSource(),results[i]));
               }
            } 
         }
         else {
            results = ffr->getWikiLinks(buffer.substr(0,pos)) ;
            if (nbResults > results.size() ) {
               nbResults = results.size();
            } 
            for ( unsigned int i = 0 ; i  < nbResults ; i ++ ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),results[i]));
            }
         }
      }
		return true;
	}
	bool planet (Message*m,Plugin*p,BotKernel*b)
	{
      const string BEGIN_HREF = "<h2><a href=\"";
      const string END_HREF = "\" title=";
      string buffer = "";
      string::size_type pos=0;
      unsigned int startFrom=5;
      unsigned int nbResults ;
      vector<string> results;
      unsigned int maxResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".planet_max_results"));
      Socket sock = Socket();
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         if (!sock.connectSock(80,"planet.fedora-fr.org","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to planet.fedora-fr.org *"));
            return true;
         }
         nbResults = Tools::strToInt(m->getPart(4));
         if (nbResults == 0 ) {
            nbResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".planet_default_results"));  
            startFrom = 4;
         }
         else  if (nbResults > maxResults ) {
            nbResults = maxResults ;
         }
         sock.sendStr("GET content/advancedsearch?SearchPageLimit=2&SearchText="+Tools::urlencode(Tools::vectorToString(m->getSplit()," ",startFrom) )+" HTTP/1.1\nHost: planet.fedora-fr.org\n\n");
         while (buffer.find("</body>")==string::npos) {
            buffer += sock.receive() ;
         } 
         do {
            pos = buffer.find(BEGIN_HREF);
            if (pos!=string::npos) {
               buffer = buffer.substr(pos+BEGIN_HREF.length());
               results.push_back(buffer.substr(0,buffer.find(END_HREF) ));
            }
         }
         while (pos !=string::npos); 
         if ( results.size() > 0 ) {
            if ( results.size() < nbResults ) {
               nbResults = results.size() ;
            }
            for (unsigned int i = 0 ; i < nbResults ; i ++ ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),results[i]));
            }
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"Aucun resultat"));
         }
      }
		return true;
	}
}
