/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file fedoraproject.cpp
 * @brief FedoraProject implementation file
 */

#include "fedoraproject.h"
#include "../botkernel.h"

/**
 * Constructor
 */ 
FedoraProject::FedoraProject(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugin in connection with fedora project";
	this->version = "0.1.1";
	this->name = "fedoraproject";
   this->bindFunction("whoowns",IN_COMMAND_HANDLER,"whoowns",0,60);
}

/**
 * Tells who owns a package using https://admin.fedoraproject.org/pkgdb/packages/name/ webpage
 * @param name Package name
 * @return Package's owner
 */
string FedoraProject::whoowns(string name) 
{
   const string URL = "https://admin.fedoraproject.org/pkgdb/packages/name/"+Tools::urlencode(name);
   const string LAST_OWNER = "<span class=\"ownerName\"><a href=\"";
   const string END_OWNER = "</a>";
   CURL *curl;  
   CURLcode result;  
   string::size_type pos;
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init();  
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, FedoraProject::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer); 
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
      if (result == CURLE_OK) {  
         pos = buffer.rfind(LAST_OWNER);
         if ( pos == string::npos ){
            return (name+" : package not found");
         }
         else {
            buffer = buffer.substr(pos+LAST_OWNER.length());
            buffer = buffer.substr(buffer.find(">")+1);
            return (name+"'s owner : "+buffer.substr(0,buffer.find(END_OWNER)));
         }
      }  
      else {  
         return (name+" : package not found");
      }  
   } 
   return "curl init error";
}

int FedoraProject::writer(char *data, size_t size, size_t nmemb,string *buffer)  
{  
   int result = 0;  
   if (buffer != NULL) {  
      buffer->append(data, size * nmemb);  
      result = size * nmemb;  
   }   
   return result;  
}

extern "C"
{
	Plugin *contruct_fedoraproject(BotKernel*b)
	{
		return new FedoraProject(b);
	}
	void destroy_fedoraproject(Plugin*p)
	{
		delete p;
	}
	bool whoowns (Message*m,Plugin*p,BotKernel*b)
	{
      FedoraProject * fp = (FedoraProject*)p;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),fp->whoowns(m->getPart(4))));
      }
		return true;
	}
}
