/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file postconnect.cpp
 * @brief PostConnect implementation file
 */

#include "postconnect.h"
#include "../botkernel.h"

/**
 * Constructor
 */
PostConnect::PostConnect(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Auto actions performed after server connection";
	this->version = "0.1.1";
	this->name = "postconnect";
	this->bindFunction("376",IN_TYPE_HANDLER,"onEndOfMOTD",0,10);
	this->bindFunction("422",IN_TYPE_HANDLER,"onEndOfMOTD",0,10); // No Motd ! (motd file missing)
	this->bindFunction("433",IN_TYPE_HANDLER,"secondaryNick",0,10);
}

extern "C"
{
	Plugin *contruct_postconnect(BotKernel*b)
	{
		return new PostConnect(b);
	}
	void destroy_postconnect(Plugin*p)
	{
		delete p;
	}
	bool getMyFirstNick(Message*m,Plugin*p,BotKernel*b)
	{
		ConfigurationFile * cf = b->getCONFF();
      b->setNick(cf->getValue("kernel.nick"));
      b->send(IRCProtocol::changeNick(cf->getValue("kernel.nick")));
		return true;
	}
	bool secondaryNick(Message*m,Plugin*p,BotKernel*b)
	{
		ConfigurationFile * cf = b->getCONFF();
		b->getSysLog()->log(INFO,"Nick "+cf->getValue("kernel.nick")+" already in use");
      b->setNick(cf->getValue("kernel.nick2"));
      b->send(IRCProtocol::changeNick(cf->getValue("kernel.nick2")));
      if ( cf->getValue(p->getName()+".retreive_nick") == "1" ) {
         if(b->addCountDown(p,getMyFirstNick,m,Tools::strToUnsignedInt(cf->getValue(p->getName()+".retreive_nick_time")),5)==NULL) {
            b->getSysLog()->log(INFO,"Couldn't launch nick take back (max countdowns reached)"); 
         }            
      }
		return true;
	}
	bool onEndOfMOTD(Message*m,Plugin*p,BotKernel*b)
	{
		vector<string> temp ;
		ConfigurationFile * cf = b->getCONFF();
		if ( cf->getValue(p->getName()+".qauth") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".qauth"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log(INFO,"QAUTH");
				b->send("PRIVMSG Q@CServe.quakenet.org :AUTH "+ temp[0] +" "+ temp[1]);
			}
			else
				b->getSysLog()->log(WARNING,"Wrong QAUTH value : "+cf->getValue(p->getName()+".qauth"));
		}
		else if ( cf->getValue(p->getName()+".qauthx") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".qauthx"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log(INFO,"QAUTH, MODE +x");
				b->send("PRIVMSG Q@CServe.quakenet.org :AUTH "+ temp[0] +" "+ temp[1]);
				b->send("MODE "+ b->getNick()+ " +x");
			}
			else
				b->getSysLog()->log(WARNING,"Wrong QAUTHX value : "+cf->getValue(p->getName()+".qauthx"));
		}
		if ( cf->getValue(p->getName()+".uauth") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".uauth"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log(INFO,"UAUTH");
				b->send("PRIVMSG X@channels.undernet.org :LOGIN "+ temp[0] +" "+ temp[1]);
			}
			else
				b->getSysLog()->log(WARNING,"Wrong UAUTH value : "+cf->getValue(p->getName()+".uauth"));
		}
		else if ( cf->getValue(p->getName()+".uauthx") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".uauthx"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log(INFO,"UAUTH, MODE +x");
				b->send("PRIVMSG X@channels.undernet.org :LOGIN "+ temp[0] +" "+ temp[1]);
				b->send("MODE "+ b->getNick() + " +x");
			}
			else
				b->getSysLog()->log(WARNING,"Wrong UAUTHX value : "+cf->getValue(p->getName()+".uauthx"));
		}
		if ( cf->getValue(p->getName()+".nsauth") != "")
		{
				b->getSysLog()->log(INFO,"NSAUTH");
				b->send("PRIVMSG NickServ :IDENTIFY "+ cf->getValue(p->getName()+".nsauth"));
		}
		temp = Tools::stringToVector(cf->getValue(p->getName()+".raw"),",",0);
		for ( unsigned int i = 0 ; i < temp.size() ; i ++ ) {
			b->getSysLog()->log(INFO,"Executing raw action : " + temp[i] );
			b->send(temp[i]);
		}
      sleep(Tools::strToInt(cf->getValue(p->getName()+".sleep")));
		temp = Tools::stringToVector(cf->getValue(p->getName()+".join"),",",0);
		for ( unsigned int i = 0 ; i < temp.size() ; i ++ ) {
			b->getSysLog()->log(INFO,"Going to join " + temp[i] );
			b->send(IRCProtocol::joinChannel(temp[i]));
		}
		return true;
	}
}
